import os
from pathlib import Path
from sys import exit
from typing import Iterable, Set

from colorama import Style

from . import DEFAULT_EXCLUDE, DEFAULT_INCLUDE, remove, search
from .config import CONFIG, Verbosity


def output(message: str, *args, verbosity: Verbosity = Verbosity.NORMAL, **kwargs):
    if verbosity <= CONFIG.verbosity:
        if verbosity >= Verbosity.VERBOSE:
            message = f"{Style.DIM}{message}{Style.RESET_ALL}"
        print(message, *args, **kwargs)


def paths_to_remove(patterns: Iterable[str], ignore: Iterable[str]) -> Set[Path]:
    paths = set()

    for path in search(*CONFIG.directories, patterns=patterns, ignore=[]):
        paths.add(path)
        output(f"Found {path}", verbosity=Verbosity.VERY_VERBOSE)

    for path in search(*CONFIG.directories, patterns=ignore, ignore=[]):
        if path in paths:
            paths.remove(path)
        output(f"Ignored {path}", verbosity=Verbosity.VERY_VERBOSE)

    return paths


def empty_directories() -> Set[Path]:
    paths = set()
    for path in search(*CONFIG.directories, patterns=["*"], ignore=DEFAULT_EXCLUDE | CONFIG.exclude):
        if path.exists() and path.is_dir():
            with os.scandir(path) as scan:
                if next(scan, None) is None:
                    paths.add(path)
                    output(f"Found {path}", verbosity=Verbosity.VERY_VERBOSE)
    return paths


def remove_paths(paths: Iterable[Path]) -> int:
    paths = set(paths)
    for path in paths:
        remove(path)
        output(f"Removing {path}", verbosity=Verbosity.VERBOSE)
    return len(paths)


def main() -> int:
    paths = paths_to_remove(DEFAULT_INCLUDE, DEFAULT_EXCLUDE) | paths_to_remove(CONFIG.include, CONFIG.exclude)
    num_removed = remove_paths(paths)

    if CONFIG.remove_empty_directories:
        empty = empty_directories()
        num_removed += remove_paths(empty)

    output(f"{num_removed} paths removed.")

    return 0


if __name__ == "__main__":
    exit(main())



import shutil
from pathlib import Path
from typing import Iterable, Set

DEFAULT_INCLUDE = {"__pycache__", "*.pyc", "*.pyo"}
DEFAULT_EXCLUDE = {".venv/**/*"}


def remove_file(file: Path) -> None:
    """Remove a file

    Args:
        path (Path): The path to the file
    """
    if file.exists():
        file.unlink()


def remove_directory(directory: Path) -> None:
    """Remove a directory

    Args:
        path (Path): The path to the directory
    """
    if directory.exists():
        shutil.rmtree(directory)


def remove(path: Path) -> None:
    """Remove a file or directory

    Args:
        path (Path): The path to the file or directory
    """
    if path.exists():
        if path.is_dir():
            remove_directory(path)
        else:
            remove_file(path)


def search(
    *directories: Path, patterns: Iterable[str] = DEFAULT_INCLUDE, ignore: Iterable[str] = DEFAULT_EXCLUDE
) -> Set[Path]:
    """Search for path patterns within directories

    Args:
        patterns (Iterable[str], optional): Patterns to glob recursively. Defaults to DEFAULT_PATTERNS.
        ignore (Iterable[str], optional): Patterns to ignore. Defaults to DEFAULT_IGNORE.

    Returns:
        Set[Path]: [description]
    """
    paths = set()
    for directory in directories:
        for pattern in patterns:
            for path in directory.rglob(pattern):
                if path.exists():
                    paths.add(path)

        for pattern in ignore:
            for path in directory.rglob(pattern):
                if path in paths:
                    paths.remove(path)

    return paths


__all__ = ["remove", "remove_file", "remove_directory", "search"]



from __future__ import annotations

import argparse
import enum
import os
from dataclasses import dataclass
from pathlib import Path
from typing import Iterable, Set, Union

import toml

CWD = Path.cwd()


class Verbosity(enum.IntEnum):
    QUIET = 0
    NORMAL = 1
    VERBOSE = 2
    VERY_VERBOSE = 3


@dataclass
class Configuration:
    directories: Set[Path]
    include: Set[str]
    exclude: Set[str]
    remove_empty_directories: bool
    verbosity: Verbosity

    def __init__(
        self,
        directories: Iterable[Union[os.PathLike, str]] = {CWD},
        include: Iterable[str] = set(),
        exclude: Iterable[str] = set(),
        remove_empty_directories: bool = False,
        quiet: bool = False,
        verbose: bool = False,
        very_verbose: bool = False,
    ) -> None:
        self.directories = set(map(lambda p: Path(p).resolve(), directories))
        self.include = set(include)
        self.exclude = set(exclude)
        self.remove_empty_directories = remove_empty_directories

        if quiet:
            self.verbosity = Verbosity.QUIET
        elif verbose:
            self.verbosity = Verbosity.VERBOSE
        elif very_verbose:
            self.verbosity = Verbosity.VERY_VERBOSE
        else:
            self.verbosity = Verbosity.NORMAL


def parse_pyproject_toml() -> dict:
    pyproject_toml = toml.load(CWD / "pyproject.toml")
    config: dict = pyproject_toml.get("tool", {}).get("dustpan", {})
    return {k.replace("-", "_"): v for k, v in config.items()}


def parse_arguments() -> dict:
    parser = argparse.ArgumentParser(description="")

    parser.add_argument("directories", type=Path, nargs="+", help="Root directories to search")

    parser.add_argument("-p", "--patterns", type=str, nargs="+", help="Additional path patterns to queue for removal")
    parser.add_argument("-i", "--ignore", type=str, nargs="+", help="Path patterns to exclude from removal")

    parser.add_argument("--remove-empty-directories", action="store_true", help="Remove all childless directories")

    verbosity = parser.add_mutually_exclusive_group()
    verbosity.add_argument("-q", "--quiet", action="store_true", help="Be quiet")
    verbosity.add_argument("-v", "--verbose", action="store_true", help="Be more verbose")
    verbosity.add_argument("-vv", "--very-verbose", action="store_true", help="Be very verbose")

    args = parser.parse_args()
    return {k: v for k, v in vars(args).items() if bool(v)}


CONFIG = Configuration(**{**parse_pyproject_toml(), **parse_arguments()})
